--------------------------------------------------------------------------------------------------------------------------
-- Marker Maker for After Effects 3.2.0.0
-- (C) 2012 あかつきみさき（みくちぃP）
-- 
-- このJob Pluginについて
--  Adobe After Effectsで実行可能なJavaScriptファイルを出力するVOCALOID3 Job Pluginです.
--  作成されたJavaScriptファイルをAfter Effects上で実行すると,VOCALOID3上で入力されたノートに基づく位置にマーカーがついたヌルレイヤーを作成します.
--  マーカーのコメント・Flashキューポイントには,ひとつは対応するノートの歌詞情報,もうひとつには母音情報を含んでいます.
-- 
-- インストール
--  VOCALOID3上でパスを設定する必要があります.
--  適当なディレクトリ(フォルダー)に置いて,VOCALOID3の「ジョブ→Jobプラグインを管理」から該当ディレクトリ(フォルダー)へパスを設定すれば完了です.
--  アンインストールはパスの削除,直接削除してください.
-- 
-- 動作環境
--  VOCALOID3 , VOCALOID3 Job Pluginが動作する環境
-- 
-- 出力したJavaScriptファイルの動作環境
--  Adobe After Effects CS3以上
--  環境によっては正しく動作しない可能性があります.
-- 
-- 使用方法
--  一つのトラック内の一つ以上のノートを含むパートを選択し,「ジョブ→Jobプラグインを実行」から実行してください.
--  ファイル名を入力してOKを押すと,vsqxファイルと同じディレクトリ(フォルダー)内に「vsqxファイル名.vsqx_入力ファイル名.jsx」という名前のJavaScriptファイルが生成されます.
--  デフォルトの場合は「vsqxファイル名.vsqx_パート名_VOCALOID3MarkerData.jsx」になっています.
--  複数のパートの同時書き出しには対応していません.一度に複数のパートで使用したい場合はパートを結合してください.
-- 
-- 出力したJavaScriptファイルの使用方法
--  After Effects上でヌルレイヤーを生成したいコンポジションを選択し,「ファイル→スクリプト→スクリプトファイルの実行」からスクリプトを実行してください.
--  選択したコンポジション内に2つ「パート名 VOCALOID3 Lyric Data (BPM ※)」「パート名 VOCALOID3 Vowel Data (BPM ※)」というレイヤー名のヌルレイヤーが作成されます.※はBPM値
-- 
-- 母音情報について
--  「パート名 VOCALOID3 Vowel Data (BPM ※)」についたマーカーの母音情報は,以下の法則に基づいて付いています.
--   清音,濁音,半濁音,小文字の"ぁ,ぃ,ぅ,ぇ,ぉ,ゃ,ゅ,ょ,ゎ","ヶ"は,それぞれ対応した母音により「a」「i」「u」「e」「o」
--   小文字の"っ",長音符,踊り字は一つ前のノートの母音
--   "ん"は,「n」
--   複数で構成される文字(きゃ)などは促音から判定
--   それ以外のものは,ノートに入力された文字がそのままマーカーのメモになります.
--   a = 1, i = 2, u = 3, e = 4, o = 5, n = 6, other = 0.
-- 
-- バージョン情報
--  2012/6/1 Ver 3.2.0.0 Update Flashキューポイントのプロパティをセットする機能の追加.
--  2012/6/1 Ver 3.1.1.0 Update 歌詞が「ki」の時に母音情報が正しく入力されないバグの修正. 促音例外分岐の分岐追加. EULAの廃止. CS6の対応.
--  2012/3/31 Ver 3.1.0.0 Update AE上でプロジェクトパネルの一番上以外でのコンポジションで実行したときにVowel Dataを読めない問題の修正.
--  2012/3/26 Ver 3.0.0.0 Update JavaScript処理速度の改善. 母音情報処理の変更. EULA処理の実装.
--  2012/3/26 Ver 2.2.1.1 Update バグの修正.
--  2012/3/14 Ver 2.2.1.0 Update pluginIDの修正, JavaScript初回起動時のEULAの追加実験(2.2.1.0では未実装)
--  2012/2/10 Ver 2.2.0.0 Update VOCALOID3発音記号の正式対応, 歌詞に発音記号が直接入力されていた時にJavaScriptが実行不可になる問題に対応.
--  2012/1/18 Ver 2.1.1.0 Update ローマ字の正式対応.
--  2012/1/14 Ver 2.1.0.0 Update AE CS5以上での動作の改善.
--  2012/1/14 Ver 2.0.0.0 Update AEでの対応バージョンの変更(CS4以前の再対応).
--  2011/12/26 Ver1.2.0.0 Update 母音情報からスライダ制御の追加. AEでの対応バージョンの変更(CS4以前の切り捨て).
--  2011/12/25 Ver1.1.0.0 Update JavaScriptに母音情報を含むマーカーのレイヤー生成機能の追加,ヘルプの追加
--  2011/12/24 Ver1.0.1.0 Update 書き出すJavaScript内のコメントの更新, AEのバージョン判定の追加
--  2011/12/16 Ver1.0.0.0 Release
--
-- プラグイン・認証スクリプト配布先記事「音のデータとAfter Effects ～VOCALOID3と連携編～」
-- http://ae-users.com/jp/resources/2012/03/音のデータとAfter-Effects-～VOCALOID3と連携編～/
-- 
-- Q & A
--  Q1. パートを選択しているのにJavaScriptファイルが出力されません.
--  A1. 選択されているパートにノートを含んでいるか確認してください.選択されているパートとはMUSICAL EDITORでアクティブになっているパートのことです.
--  Q2. 出力したJavaScriptファイルの歌詞データ(マーカーのメモ)がおかしい.
--  A2. 作成されたJavaScriptファイルの歌詞情報はVOCALOID3上で実際にノートに入力されているデータを参照しています.
--         例えば,「カレーはおいしかった」という歌詞だと,一例として「かれえわおいしかた」といったように,VOCALOIDに正しく歌わせるために入力された歌詞になっています.
--  Q3. 日本語以外の歌詞の母音情報が正しく入力されていない.
--  A3. 母音情報は歌詞が日本語の時にのみ有効です.
--------------------------------------------------------------------------------------------------------------------------

-- プラグインマニフェスト関数
function manifest()
    myManifest = {
        name = "Marker Maker for After Effects",
        comment = "After Effectsで動作するマーカー付きヌルレイヤー生成スクリプトを出力します",
        author = "Misaki Akatsuki_102",
        pluginID = "{C77C1315-42B1-4e53-8959-8FF8241A2942}",
        pluginVersion = "3.2.0.0",
        apiVersion = "3.0.0.1"
    }

    return myManifest
end

-- VOCALOID3 Jobプラグインスクリプトのエントリポイント
function main(processParam, envParam)
    
    local pluginName = "Marker Maker for After Effects"
    local pluginVer = "3.2.0.0"
-- 実行時に渡されたパラメータを取得
    local songPosTick = processParam.songPosTick
    local note = {}
    local noteEx = {}
    local noteExList = {}
    local noteCount
    local retCode
    local idx
    local musicalPart = {}
    local musicalSinger = {}
    local msg
    local tempo = {}
    local tempoList = {}
    local tempoNum
    local timeSig = {}
    local timeSigList = {}
    local timeSigNum
    local seqname = {} -- シーケンスネーム
    local partname -- パートネーム
    local lyric -- 歌詞
    local bpm -- BPM
    local min -- 秒数

-- Musicalパート情報を取得
    retCode, musicalPart, note = VSGetMusicalPart()
    partname = musicalPart.name
    
-- パートの中のノートを取得してノートイベント配列へ格納
    VSSeekToBeginNote()
    idx = 1
    retCode, noteEx = VSGetNextNoteEx()
    while (retCode == 1) do
        noteExList[idx] = noteEx
        retCode, noteEx = VSGetNextNoteEx()
        idx = idx + 1
    end

-- 選択したパートに入っているノートを読み込み
    noteCount = table.getn(noteExList)
-- ノート = 0のエラー処理
    if (noteCount == 0) then
        VSMessageBox("ノートを含むパートを選択してください.", 0)
        return 0
    end

-- 拍子の取得
    VSSeekToBeginTimeSig()
    idx = 1
    retCode, timeSig = VSGetNextTimeSig()
    while (retCode == 1) do
        timeSigList[idx] = timeSig
        retCode, timeSig = VSGetNextTimeSig()
        idx = idx + 1
    end
    timeSigNum = table.getn(timeSigList)

-- テンポの取得
    VSSeekToBeginTempo()
    idx = 1
    retCode, tempo = VSGetNextTempo()
    while (retCode == 1) do
        tempoList[idx] = tempo
        retCode, tempo = VSGetNextTempo()
        idx = idx + 1
    end
    tempoNum = table.getn(tempoList)
    bpm = tempoList[tempoNum].tempo

-- ファイルパス取得
    local sequencePath = VSGetSequencePath()

-- パラメータ入力ダイアログのウィンドウタイトルを設定
    VSDlgSetDialogTitle("名前を付けて保存(ファイル名.jsxの保存)")
    local dlgStatus  = VSDlgAddField(field)
    local field = {}
    field.name = "fname"
    field.caption = "ファイル名 (既存のファイルは上書きされます)"
    field.initialVal = partname .. "_VOCALOID3MarkerData"
    field.type = 3
    dlgStatus = VSDlgAddField(field)

-- パラメータ入力ダイアログを表示
    dlgStatus = VSDlgDoModal()
    if  (dlgStatus ~= 1) then
    -- OKボタンが押されなかったら終了
        return 0
    end

    local filename
    dlgStatus, filename = VSDlgGetStringValue("fname")

-- JavaScript出力
    io.output(sequencePath .. "_" .. filename .. ".jsx")

    io.write( "//////////////////////////////////////////////////////////////////////////////////////////////////\n" )
    io.write( "// VOCALOID3 Job Plugin " .. pluginName .. " " .. pluginVer .. "\n" )
    io.write( "// (C) あかつきみさき(みくちぃP)\n" )
    io.write( "// \n" )
    io.write( "// このスクリプトについて\n" )
    io.write( "//  VOCALOID3 Job Plugin 「".. pluginName .."」で出力されたものです.\n" )
    io.write( "//  VOCALOID3上で入力されたノートに基づく位置にマーカーがついたヌルレイヤーを作成します.\n" )
    io.write( "//  マーカーのメモには,ひとつは対応するノートの歌詞情報,もうひとつには母音情報を含んでいます.\n" )
    io.write( "//  詳しくは".. pluginName .."のヘルプをご覧ください.\n" )
    io.write( "// \n" )
    io.write( "// 使用方法\n" )
    io.write( "//  After Effects上でヌルレイヤーを生成したいコンポジションを選択し,「ファイル→スクリプト→スクリプトファイルの実行」からスクリプトを実行してください.\n" )
    io.write( "//  選択したコンポジション内に2つ「パート名 VOCALOID3 Lyric Data (BPM ※)」「パート名 VOCALOID3 Vowel Data (BPM ※)」というレイヤー名のヌルレイヤーが作成されます.※はBPM値\n" )
    io.write( "// \n" )
    io.write( "// 動作環境\n" )
    io.write( "//  Adobe After Effects CS3以上\n" )
    io.write( "//  環境によっては正しく動作しない可能性があります.\n" )
    io.write( "//  うまく動作しない場合,「#target AfterFx」の一文を削除すると動作する場合があります.\n" )
    io.write( "// \n" )
    io.write( "// プラグイン配布先記事「音のデータとAfter Effects ～VOCALOID3と連携編～」\n" )
    io.write( "//  http://ae-users.com/jp/resources/2012/03/音のデータとAfter-Effects-～VOCALOID3と連携編～/\n" )
    io.write( "//////////////////////////////////////////////////////////////////////////////////////////////////\n" )
    io.write( "\n" )
    io.write( "#target AfterFx\n" )
    io.write( '\n' )
    
    io.write( 'var myGlobals = new Object();\n' )
    io.write( 'myGlobals.scriptName ="'.. pluginName ..'";\n' )
    io.write( 'myGlobals.scriptVersionNumber ="' .. pluginVer .. '";\n' )
    io.write( 'myGlobals.scriptVersion = "Version "+myGlobals.scriptVersionNumber;\n' )
    io.write( 'myGlobals.scriptURLName ="MarkerMakerforAfterEffects";\n' )
    io.write( 'myGlobals.name = "' .. filename ..  '";\n' )
    io.write( 'myGlobals.partname = "' .. partname ..  '";\n' )
    io.write( '\n' )
    
    io.write( 'if (parseFloat(app.version) < 10) {\n' )
    io.write( '  myGlobals.sliderName = "スライダ";\n' )
    io.write( '  myGlobals.sliderEffectName = myGlobals.sliderName + "制御";\n' )
    io.write( '} else {\n' )
    io.write( '  myGlobals.sliderName = "スライダー";\n' )
    io.write( '  myGlobals.sliderEffectName = myGlobals.sliderName + "制御";\n' )
    io.write( '}\n' )
    io.write( '\n' )
    io.write( 'writeLn("'.. pluginName ..'");\n' )
    io.write( 'writeLn("Load setting...");\n' )
    io.write( 'writeLn("");\n' )
    io.write( "if (parseFloat(app.version) < 7) {\n" )
    io.write( "  alert(\"This script requires After Effects 7.0 or greater.\");\n" )
    io.write( '  writeLn("'.. pluginName ..'");\n' )
    io.write( '  writeLn("uncompleted");\n' )
    io.write( '  writeLn("");\n' )
    io.write( "} else {\n" )
    io.write( "  if (!app.project || !app.project.activeItem) {\n" )
    io.write( "    alert(\"Select composition.\");\n" )
    io.write( '    writeLn("'.. pluginName ..'");\n' )
    io.write( '    writeLn("uncompleted");\n' )
    io.write( '    writeLn("");\n' )
    io.write( "  }else{\n" )
    
-- 歌詞情報付きマーカー記入
    io.write( "    var V3Lyric = app.project.activeItem.layers.addNull();\n" )
    io.write( "    V3Lyric.name = myGlobals.partname + \" VOCALOID3 Lyric Data (BPM " .. bpm .. ")\";\n" )
    io.write( "    var activeItem = app.project.activeItem;\n" )
    io.write( '    writeLn("");\n' )
    io.write( '    writeLn("Loading Lyric Data...");\n')
    io.write( '    writeLn("0%\");\n' )
    io.write( '\n' )

    for idx = 1, noteCount do
        lyric = noteExList[idx].lyric
        min = noteExList[idx].posTick / 480 * 60 / bpm

-- 発音記号エラー回避例外分岐
        if (lyric == 'h\\') then
            lyric = "h"
        elseif (lyric == 'p\\') then
            lyric = "h"
        elseif (lyric == "p\\'") then
            lyric = "f"
        elseif (lyric == 'N\\') then
            lyric = "n"
-- 英語
        elseif (lyric == '@') then
            lyric = "a"
        elseif (lyric == 'i:') then
            lyric = "i"
        elseif (lyric == '{') then
            lyric = "a"
        elseif (lyric == 'O:') then
            lyric = "a"
        elseif (lyric == 'u:') then
            lyric = "o"
        elseif (lyric == '@r') then
            lyric = "a"
        elseif (lyric == '@U') then
            lyric = "o"
        elseif (lyric == 'l@') then
            lyric = "i"
        elseif (lyric == 'e@') then
            lyric = "e"
        elseif (lyric == 'U@') then
            lyric = "o"
        elseif (lyric == 'O@') then
            lyric = "u"
        elseif (lyric == 'Q@') then
            lyric = "a"
        end
        
        io.write( '\n' )
        io.write( "    var myLyric" .. idx .. " = new MarkerValue(\"" .. lyric .. "\");\n" )
        io.write( "    V3Lyric.property(\"Marker\").setValueAtTime(" .. min .. ", myLyric" .. idx .. ");\n" )
        io.write( "    var curMarker = V3Lyric.property(\"Marker\").keyValue(" .. idx .. ");\n" )
        io.write( "    curMarker.cuePointName = myLyric" .. idx .. ";\n" )
        io.write( "    curMarker.eventCuePoint = true;\n" )
        io.write( "    V3Lyric.property(\"Marker\").setValueAtKey(" .. idx .. ",curMarker);\n" )
        
-- パーセントデータ挿入
        par = idx / noteCount * 100
        par = math.floor(par)
        
        io.write( '    writeLn("");\n' )
        io.write( "    writeLn(\"Loading Lyric Data...\");\n" )
        io.write( "    writeLn(\"" .. par .. "%\");\n" )
    end
        io.write( "\n" )

-- 母音情報付きマーカー記入
    io.write( "    var V3Vowel = app.project.activeItem.layers.addNull();\n" )
    io.write( "    V3Vowel.name = myGlobals.partname + \" VOCALOID3 Vowel Data (BPM " .. bpm .. ")\";\n" )
    io.write( "    var activeItem = app.project.activeItem;\n" )
    io.write( "    var firstLayer = app.project.activeItem.layer(1);\n" )
    io.write( '\n' )
    io.write( '    writeLn("");\n' )
    io.write( '    writeLn("Loading Vowel Data...");\n')
    io.write( '    writeLn("0%\");\n' )
    io.write( "    app.project.activeItem.layer(1).property(\"エフェクト\").addProperty(myGlobals.sliderEffectName);\n" )
    
    for idx = 1, noteCount do
        lyric = noteExList[idx].lyric
        min = noteExList[idx].posTick / 480 * 60 / bpm
-- 母音判別
-- 促音例外分岐
        if (lyric == "っ") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "xtu") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "ltu") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
-- 長音符例外分岐
        if (lyric == "ー") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "-") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
-- 踊り字例外分岐
        if (lyric == "々") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "ヽ") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "ゝ") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "ゞ") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "ヾ") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
        if (lyric == "〻") then
            idx2 = idx -1
            lyric = noteExList[idx2].lyric
        end
-- 発音記号分岐
        if (lyric == "a") then
            vowel = "a"
        elseif (lyric == "i") then
            vowel = "i"
        elseif (lyric == "M") then
            vowel = "u"
        elseif (lyric == "e") then
            vowel = "e"
        elseif (lyric == "o") then
            vowel = "o"
        elseif (lyric == "k") then
            vowel = lyric
        elseif (lyric == "k'") then
            vowel = lyric
        elseif (lyric == "g") then
            vowel = lyric
        elseif (lyric == "g'") then
            vowel = lyric
        elseif (lyric == "N") then
            vowel = "g"
        elseif (lyric == "N'") then
            vowel = "g"
        elseif (lyric == "s") then
            vowel = lyric
        elseif (lyric == "S") then
            vowel = "s"
        elseif (lyric == "z") then
            vowel = lyric
        elseif (lyric == "Z") then
            vowel = "j"
        elseif (lyric == "dz") then
            vowel = "z"
        elseif (lyric == "dZ") then
            vowel = "j"
        elseif (lyric == "t") then
            vowel = lyric
        elseif (lyric == "t'") then
            vowel = "t"
        elseif (lyric == "ts") then
            vowel = "t"
        elseif (lyric == "tS") then
            vowel = "t"
        elseif (lyric == "d") then
            vowel = lyric
        elseif (lyric == "d'") then
            vowel = "d"
        elseif (lyric == "n") then
            vowel = lyric
        elseif (lyric == "J") then
            vowel = "n"
        elseif (lyric == "h") then
            vowel = lyric
        elseif (lyric == 'h\\') then
            vowel = "h"
        elseif (lyric == "C") then
            vowel = "h"
        elseif (lyric == 'p\\') then
            vowel = "h"
        elseif (lyric == "p\\'") then
            vowel = "f"
        elseif (lyric == "b") then
            vowel = lyric
        elseif (lyric == "b'") then
            vowel = "b"
        elseif (lyric == "p") then
            vowel = lyric
        elseif (lyric == "p'") then
            vowel = "p"
        elseif (lyric == "m") then
            vowel = lyric
        elseif (lyric == "m'") then
            vowel = "m"
        elseif (lyric == "j") then
            vowel = "y"
        elseif (lyric == "4") then
            vowel = "r"
        elseif (lyric == "4'") then
            vowel = "r"
        elseif (lyric == "w") then
            vowel = lyric
        elseif (lyric == "N\\") then
            vowel = "n"

-- 清音
        elseif (lyric == "あ") then
            vowel = "a"
        elseif (lyric == "い") then
            vowel = "i"
        elseif (lyric == "う") then
            vowel = "u"
        elseif (lyric == "え") then
            vowel = "e"
        elseif (lyric == "お") then
            vowel = "o"
        elseif (lyric == "か") then
            vowel = "a"
        elseif (lyric == "き") then
            vowel = "i"
        elseif (lyric == "く") then
            vowel = "u"
        elseif (lyric == "け") then
            vowel = "e"
        elseif (lyric == "こ") then
            vowel = "o"
        elseif (lyric == "さ") then
            vowel = "a"
        elseif (lyric == "し") then
            vowel = "i"
        elseif (lyric == "す") then
            vowel = "u"
        elseif (lyric == "せ") then
            vowel = "e"
        elseif (lyric == "そ") then
            vowel = "o"
        elseif (lyric == "た") then
            vowel = "a"
        elseif (lyric == "ち") then
            vowel = "i"
        elseif (lyric == "つ") then
            vowel = "u"
        elseif (lyric == "て") then
            vowel = "e"
        elseif (lyric == "と") then
            vowel = "o"
        elseif (lyric == "な") then
            vowel = "a"
        elseif (lyric == "に") then
            vowel = "i"
        elseif (lyric == "ぬ") then
            vowel = "u"
        elseif (lyric == "ね") then
            vowel = "e"
        elseif (lyric == "の") then
            vowel = "o"
        elseif (lyric == "は") then
            vowel = "a"
        elseif (lyric == "ひ") then
            vowel = "i"
        elseif (lyric == "ふ") then
            vowel = "u"
        elseif (lyric == "へ") then
            vowel = "e"
        elseif (lyric == "ほ") then
            vowel = "o"
        elseif (lyric == "ま") then
            vowel = "a"
        elseif (lyric == "み") then
            vowel = "i"
        elseif (lyric == "む") then
            vowel = "u"
        elseif (lyric == "め") then
            vowel = "e"
        elseif (lyric == "も") then
            vowel = "o"
        elseif (lyric == "や") then
            vowel = "a"
        elseif (lyric == "ゐ") then
            vowel = "i"
        elseif (lyric == "ゆ") then
            vowel = "u"
        elseif (lyric == "ゑ") then
            vowel = "e"
        elseif (lyric == "よ") then
            vowel = "o"
        elseif (lyric == "ら") then
            vowel = "a"
        elseif (lyric == "り") then
            vowel = "i"
        elseif (lyric == "る") then
            vowel = "u"
        elseif (lyric == "れ") then
            vowel = "e"
        elseif (lyric == "ろ") then
            vowel = "o"
        elseif (lyric == "わ") then
            vowel = "a"
        elseif (lyric == "を") then
            vowel = "o"
        elseif (lyric == "ん") then
            vowel = "n"
        elseif (lyric == "ヶ") then
            vowel = "a"
-- 濁音
        elseif (lyric == "が") then
            vowel = "a"
        elseif (lyric == "ぎ") then
            vowel = "i"
        elseif (lyric == "ぐ") then
            vowel = "u"
        elseif (lyric == "げ") then
            vowel = "e"
        elseif (lyric == "ご") then
            vowel = "o"
        elseif (lyric == "ざ") then
            vowel = "a"
        elseif (lyric == "じ") then
            vowel = "i"
        elseif (lyric == "ず") then
            vowel = "u"
        elseif (lyric == "ぜ") then
            vowel = "e"
        elseif (lyric == "ぞ") then
            vowel = "o"
        elseif (lyric == "だ") then
            vowel = "a"
        elseif (lyric == "ぢ") then
            vowel = "i"
        elseif (lyric == "づ") then
            vowel = "u"
        elseif (lyric == "で") then
            vowel = "e"
        elseif (lyric == "ど") then
            vowel = "o"
        elseif (lyric == "ば") then
            vowel = "a"
        elseif (lyric == "び") then
            vowel = "i"
        elseif (lyric == "ぶ") then
            vowel = "u"
        elseif (lyric == "べ") then
            vowel = "e"
        elseif (lyric == "ぼ") then
            vowel = "o"
        elseif (lyric == "ヴ") then
            vowel = "u"
-- 半濁音
        elseif (lyric == "ぱ") then
            vowel = "a"
        elseif (lyric == "ぴ") then
            vowel = "i"
        elseif (lyric == "ぷ") then
            vowel = "u"
        elseif (lyric == "ぺ") then
            vowel = "e"
        elseif (lyric == "ぽ") then
            vowel = "o"
-- 小文字
        elseif (lyric == "ぁ") then
            vowel = "a"
        elseif (lyric == "ぃ") then
            vowel = "i"
        elseif (lyric == "ぅ") then
            vowel = "u"
        elseif (lyric == "ぇ") then
            vowel = "e"
        elseif (lyric == "ぉ") then
            vowel = "o"
        elseif (lyric == "ゃ") then
            vowel = "a"
        elseif (lyric == "ゅ") then
            vowel = "u"
        elseif (lyric == "ょ") then
            vowel = "o"
        elseif (lyric == "ゎ") then
            vowel = "a"
-- 複数文字(清音)
        elseif (lyric == "きゃ") then
            vowel = "a"
        elseif (lyric == "きぃ") then
            vowel = "i"
        elseif (lyric == "きゅ") then
            vowel = "u"
        elseif (lyric == "きぇ") then
            vowel = "e"
        elseif (lyric == "きょ") then
            vowel = "o"
        elseif (lyric == "くぁ") then
            vowel = "a"
        elseif (lyric == "くぃ") then
            vowel = "i"
        elseif (lyric == "くゅ") then
            vowel = "u"
        elseif (lyric == "くぇ") then
            vowel = "e"
        elseif (lyric == "くぉ") then
            vowel = "o"
        elseif (lyric == "しゃ") then
            vowel = "a"
        elseif (lyric == "しぃ") then
            vowel = "i"
        elseif (lyric == "しゅ") then
            vowel = "u"
        elseif (lyric == "しぇ") then
            vowel = "e"
        elseif (lyric == "しょ") then
            vowel = "o"
        elseif (lyric == "ちゃ") then
            vowel = "a"
        elseif (lyric == "ちぃ") then
            vowel = "i"
        elseif (lyric == "ちゅ") then
            vowel = "u"
        elseif (lyric == "ちぇ") then
            vowel = "e"
        elseif (lyric == "ちょ") then
            vowel = "o"
        elseif (lyric == "てゃ") then
            vowel = "a"
        elseif (lyric == "てぃ") then
            vowel = "i"
        elseif (lyric == "てゅ") then
            vowel = "u"
        elseif (lyric == "てぇ") then
            vowel = "e"
        elseif (lyric == "てょ") then
            vowel = "o"
        elseif (lyric == "にゃ") then
            vowel = "a"
        elseif (lyric == "にぃ") then
            vowel = "i"
        elseif (lyric == "にゅ") then
            vowel = "u"
        elseif (lyric == "にぇ") then
            vowel = "e"
        elseif (lyric == "にょ") then
            vowel = "o"
        elseif (lyric == "ひゃ") then
            vowel = "a"
        elseif (lyric == "ひぃ") then
            vowel = "i"
        elseif (lyric == "ひゅ") then
            vowel = "u"
        elseif (lyric == "ひぇ") then
            vowel = "e"
        elseif (lyric == "ひょ") then
            vowel = "o"
        elseif (lyric == "みゃ") then
            vowel = "a"
        elseif (lyric == "みぃ") then
            vowel = "i"
        elseif (lyric == "みゅ") then
            vowel = "u"
        elseif (lyric == "みぇ") then
            vowel = "e"
        elseif (lyric == "みょ") then
            vowel = "o"
        elseif (lyric == "りゃ") then
            vowel = "a"
        elseif (lyric == "りぃ") then
            vowel = "i"
        elseif (lyric == "りゅ") then
            vowel = "u"
        elseif (lyric == "りぇ") then
            vowel = "e"
        elseif (lyric == "りょ") then
            vowel = "o"
        elseif (lyric == "うぁ") then
            vowel = "a"
        elseif (lyric == "うぃ") then
            vowel = "i"
        elseif (lyric == "うぅ") then
            vowel = "u"
        elseif (lyric == "うぇ") then
            vowel = "e"
        elseif (lyric == "うぉ") then
            vowel = "o"
-- 複数文字(濁音)
        elseif (lyric == "ヴぁ") then
            vowel = "a"
        elseif (lyric == "ヴぃ") then
            vowel = "i"
        elseif (lyric == "ヴぅ") then
            vowel = "u"
        elseif (lyric == "ヴぇ") then
            vowel = "e"
        elseif (lyric == "ヴぉ") then
            vowel = "o"
        elseif (lyric == "ぎゃ") then
            vowel = "a"
        elseif (lyric == "ぎぃ") then
            vowel = "i"
        elseif (lyric == "ぎゅ") then
            vowel = "u"
        elseif (lyric == "ぎぇ") then
            vowel = "e"
        elseif (lyric == "ぎょ") then
            vowel = "o"
        elseif (lyric == "ぐぁ") then
            vowel = "a"
        elseif (lyric == "ぐぃ") then
            vowel = "i"
        elseif (lyric == "ぐゅ") then
            vowel = "u"
        elseif (lyric == "ぐぇ") then
            vowel = "e"
        elseif (lyric == "ぐぉ") then
            vowel = "o"
        elseif (lyric == "じゃ") then
            vowel = "a"
        elseif (lyric == "じぃ") then
            vowel = "i"
        elseif (lyric == "じゅ") then
            vowel = "u"
        elseif (lyric == "じぇ") then
            vowel = "e"
        elseif (lyric == "じょ") then
            vowel = "o"
        elseif (lyric == "ぢゃ") then
            vowel = "a"
        elseif (lyric == "ぢぃ") then
            vowel = "i"
        elseif (lyric == "ぢゅ") then
            vowel = "u"
        elseif (lyric == "ぢぇ") then
            vowel = "e"
        elseif (lyric == "ぢょ") then
            vowel = "o"
        elseif (lyric == "でゃ") then
            vowel = "a"
        elseif (lyric == "でぃ") then
            vowel = "i"
        elseif (lyric == "でゅ") then
            vowel = "u"
        elseif (lyric == "でぇ") then
            vowel = "e"
        elseif (lyric == "でょ") then
            vowel = "o"
        elseif (lyric == "びゃ") then
            vowel = "a"
        elseif (lyric == "びぃ") then
            vowel = "i"
        elseif (lyric == "びゅ") then
            vowel = "u"
        elseif (lyric == "びぇ") then
            vowel = "e"
        elseif (lyric == "びょ") then
            vowel = "o"
-- 複数文字(半濁音)
        elseif (lyric == "ぴゃ") then
            vowel = "a"
        elseif (lyric == "ぴぃ") then
            vowel = "i"
        elseif (lyric == "ぴゅ") then
            vowel = "u"
        elseif (lyric == "ぴぇ") then
            vowel = "e"
        elseif (lyric == "ぴょ") then
            vowel = "o"
-- 2バイト英字処理(清書)
        elseif (lyric == "ａ") then
            vowel = "a"
        elseif (lyric == "ｉ") then
            vowel = "i"
        elseif (lyric == "ｕ") then
            vowel = "u"
        elseif (lyric == "ｅ") then
            vowel = "e"
        elseif (lyric == "ｏ") then
            vowel = "o"
        elseif (lyric == "ｎ") then
            vowel = "n"
-- ローマ字処理(清書)
        elseif (lyric == "a") then
            vowel = "a"
        elseif (lyric == "i") then
            vowel = "i"
        elseif (lyric == "u") then
            vowel = "u"
        elseif (lyric == "e") then
            vowel = "e"
        elseif (lyric == "o") then
            vowel = "o"
        elseif (lyric == "ka") then
            vowel = "a"
        elseif (lyric == "ki") then
            vowel = "i"
        elseif (lyric == "ku") then
            vowel = "u"
        elseif (lyric == "ke") then
            vowel = "e"
        elseif (lyric == "ko") then
            vowel = "o"
        elseif (lyric == "sa") then
            vowel = "a"
        elseif (lyric == "si") then
            vowel = "i"
        elseif (lyric == "shi") then
            vowel = "i"
        elseif (lyric == "su") then
            vowel = "u"
        elseif (lyric == "se") then
            vowel = "e"
        elseif (lyric == "so") then
            vowel = "o"
        elseif (lyric == "ta") then
            vowel = "a"
        elseif (lyric == "ti") then
            vowel = "i"
        elseif (lyric == "chi") then
            vowel = "i"
        elseif (lyric == "tu") then
            vowel = "u"
        elseif (lyric == "te") then
            vowel = "e"
        elseif (lyric == "to") then
            vowel = "o"
        elseif (lyric == "na") then
            vowel = "a"
        elseif (lyric == "ni") then
            vowel = "i"
        elseif (lyric == "nu") then
            vowel = "u"
        elseif (lyric == "ne") then
            vowel = "e"
        elseif (lyric == "no") then
            vowel = "o"
        elseif (lyric == "ha") then
            vowel = "a"
        elseif (lyric == "hi") then
            vowel = "i"
        elseif (lyric == "hu") then
            vowel = "u"
        elseif (lyric == "fu") then
            vowel = "u"
        elseif (lyric == "he") then
            vowel = "e"
        elseif (lyric == "ho") then
            vowel = "o"
        elseif (lyric == "ma") then
            vowel = "a"
        elseif (lyric == "mi") then
            vowel = "i"
        elseif (lyric == "mu") then
            vowel = "u"
        elseif (lyric == "me") then
            vowel = "e"
        elseif (lyric == "mo") then
            vowel = "o"
        elseif (lyric == "ya") then
            vowel = "a"
        elseif (lyric == "yi") then
            vowel = "i"
        elseif (lyric == "yu") then
            vowel = "u"
        elseif (lyric == "ye") then
            vowel = "e"
        elseif (lyric == "yo") then
            vowel = "o"
        elseif (lyric == "ra") then
            vowel = "a"
        elseif (lyric == "ri") then
            vowel = "i"
        elseif (lyric == "ru") then
            vowel = "u"
        elseif (lyric == "re") then
            vowel = "e"
        elseif (lyric == "ro") then
            vowel = "o"
        elseif (lyric == "wa") then
            vowel = "a"
        elseif (lyric == "wo") then
            vowel = "o"
        elseif (lyric == "n") then
            vowel = "n"
        elseif (lyric == "nn") then
            vowel = "n"
-- ローマ字処理(濁音)
        elseif (lyric == "ga") then
            vowel = "a"
        elseif (lyric == "gi") then
            vowel = "i"
        elseif (lyric == "gu") then
            vowel = "u"
        elseif (lyric == "ge") then
            vowel = "e"
        elseif (lyric == "go") then
            vowel = "o"
        elseif (lyric == "za") then
            vowel = "a"
        elseif (lyric == "zi") then
            vowel = "i"
        elseif (lyric == "ji") then
            vowel = "i"
        elseif (lyric == "zu") then
            vowel = "u"
        elseif (lyric == "ze") then
            vowel = "e"
        elseif (lyric == "zo") then
            vowel = "o"
        elseif (lyric == "da") then
            vowel = "a"
        elseif (lyric == "di") then
            vowel = "i"
        elseif (lyric == "du") then
            vowel = "u"
        elseif (lyric == "de") then
            vowel = "e"
        elseif (lyric == "do") then
            vowel = "o"
        elseif (lyric == "ba") then
            vowel = "a"
        elseif (lyric == "bi") then
            vowel = "i"
        elseif (lyric == "bu") then
            vowel = "u"
        elseif (lyric == "be") then
            vowel = "e"
        elseif (lyric == "bo") then
            vowel = "o"
-- ローマ字処理(半濁音)
        elseif (lyric == "pa") then
            vowel = "a"
        elseif (lyric == "pi") then
            vowel = "i"
        elseif (lyric == "pu") then
            vowel = "u"
        elseif (lyric == "pe") then
            vowel = "e"
        elseif (lyric == "po") then
            vowel = "o"
-- ローマ字処理(小文字)
        elseif (lyric == "xa") then
            vowel = "a"
        elseif (lyric == "xi") then
            vowel = "i"
        elseif (lyric == "xu") then
            vowel = "u"
        elseif (lyric == "xe") then
            vowel = "e"
        elseif (lyric == "xo") then
            vowel = "o"
        elseif (lyric == "xya") then
            vowel = "a"
        elseif (lyric == "xyu") then
            vowel = "u"
        elseif (lyric == "xyo") then
            vowel = "o"
        elseif (lyric == "xwa") then
            vowel = "a"
        elseif (lyric == "la") then
            vowel = "a"
        elseif (lyric == "li") then
            vowel = "i"
        elseif (lyric == "lu") then
            vowel = "u"
        elseif (lyric == "le") then
            vowel = "e"
        elseif (lyric == "lo") then
            vowel = "o"
        elseif (lyric == "lya") then
            vowel = "a"
        elseif (lyric == "lyu") then
            vowel = "u"
        elseif (lyric == "lyo") then
            vowel = "o"
        elseif (lyric == "lwa") then
            vowel = "a"
-- ローマ字処理(複数文字(清音))
        elseif (lyric == "kya") then
            vowel = "a"
        elseif (lyric == "kyi") then
            vowel = "i"
        elseif (lyric == "kyu") then
            vowel = "u"
        elseif (lyric == "kye") then
            vowel = "e"
        elseif (lyric == "kyo") then
            vowel = "o"
        elseif (lyric == "qa") then
            vowel = "a"
        elseif (lyric == "qi") then
            vowel = "i"
        elseif (lyric == "qu") then
            vowel = "u"
        elseif (lyric == "qe") then
            vowel = "e"
        elseif (lyric == "qo") then
            vowel = "o"
        elseif (lyric == "sha") then
            vowel = "a"
        elseif (lyric == "shu") then
            vowel = "u"
        elseif (lyric == "she") then
            vowel = "e"
        elseif (lyric == "sho") then
            vowel = "o"
        elseif (lyric == "sya") then
            vowel = "a"
        elseif (lyric == "syi") then
            vowel = "i"
        elseif (lyric == "syu") then
            vowel = "u"
        elseif (lyric == "sye") then
            vowel = "e"
        elseif (lyric == "syo") then
            vowel = "o"
        elseif (lyric == "cha") then
            vowel = "a"
        elseif (lyric == "chu") then
            vowel = "u"
        elseif (lyric == "che") then
            vowel = "e"
        elseif (lyric == "cho") then
            vowel = "o"
        elseif (lyric == "tya") then
            vowel = "a"
        elseif (lyric == "tyi") then
            vowel = "i"
        elseif (lyric == "tyu") then
            vowel = "u"
        elseif (lyric == "tye") then
            vowel = "e"
        elseif (lyric == "tyo") then
            vowel = "o"
        elseif (lyric == "tha") then
            vowel = "a"
        elseif (lyric == "thi") then
            vowel = "i"
        elseif (lyric == "thu") then
            vowel = "u"
        elseif (lyric == "the") then
            vowel = "e"
        elseif (lyric == "tho") then
            vowel = "o"
        elseif (lyric == "nya") then
            vowel = "a"
        elseif (lyric == "nyi") then
            vowel = "i"
        elseif (lyric == "nyu") then
            vowel = "u"
        elseif (lyric == "nye") then
            vowel = "e"
        elseif (lyric == "nyo") then
            vowel = "o"
        elseif (lyric == "hya") then
            vowel = "a"
        elseif (lyric == "hyi") then
            vowel = "i"
        elseif (lyric == "hyu") then
            vowel = "u"
        elseif (lyric == "hye") then
            vowel = "e"
        elseif (lyric == "hyo") then
            vowel = "o"
        elseif (lyric == "mya") then
            vowel = "a"
        elseif (lyric == "myi") then
            vowel = "i"
        elseif (lyric == "myu") then
            vowel = "u"
        elseif (lyric == "mye") then
            vowel = "e"
        elseif (lyric == "myo") then
            vowel = "o"
        elseif (lyric == "rya") then
            vowel = "a"
        elseif (lyric == "ryi") then
            vowel = "i"
        elseif (lyric == "ryu") then
            vowel = "u"
        elseif (lyric == "rye") then
            vowel = "e"
        elseif (lyric == "ryo") then
            vowel = "o"
        elseif (lyric == "wha") then
            vowel = "a"
        elseif (lyric == "whi") then
            vowel = "i"
        elseif (lyric == "whu") then
            vowel = "u"
        elseif (lyric == "whe") then
            vowel = "e"
        elseif (lyric == "who") then
            vowel = "o"
-- ローマ字処理(複数文字(濁音))
        elseif (lyric == "va") then
            vowel = "a"
        elseif (lyric == "vi") then
            vowel = "i"
        elseif (lyric == "vu") then
            vowel = "u"
        elseif (lyric == "ve") then
            vowel = "e"
        elseif (lyric == "vo") then
            vowel = "o"
        elseif (lyric == "gya") then
            vowel = "a"
        elseif (lyric == "gyi") then
            vowel = "i"
        elseif (lyric == "gyu") then
            vowel = "u"
        elseif (lyric == "gye") then
            vowel = "e"
        elseif (lyric == "gyo") then
            vowel = "o"
        elseif (lyric == "gwa") then
            vowel = "a"
        elseif (lyric == "gwi") then
            vowel = "i"
        elseif (lyric == "gwu") then
            vowel = "u"
        elseif (lyric == "gwe") then
            vowel = "e"
        elseif (lyric == "gwo") then
            vowel = "o"
        elseif (lyric == "ja") then
            vowel = "a"
        elseif (lyric == "ji") then
            vowel = "i"
        elseif (lyric == "ju") then
            vowel = "u"
        elseif (lyric == "je") then
            vowel = "e"
        elseif (lyric == "jo") then
            vowel = "o"
        elseif (lyric == "jya") then
            vowel = "a"
        elseif (lyric == "jyi") then
            vowel = "i"
        elseif (lyric == "jyu") then
            vowel = "u"
        elseif (lyric == "jye") then
            vowel = "e"
        elseif (lyric == "jyo") then
            vowel = "o"
        elseif (lyric == "zya") then
            vowel = "a"
        elseif (lyric == "zyi") then
            vowel = "i"
        elseif (lyric == "zyu") then
            vowel = "u"
        elseif (lyric == "zye") then
            vowel = "e"
        elseif (lyric == "zyo") then
            vowel = "o"
        elseif (lyric == "dya") then
            vowel = "a"
        elseif (lyric == "dyi") then
            vowel = "i"
        elseif (lyric == "dyu") then
            vowel = "u"
        elseif (lyric == "dye") then
            vowel = "e"
        elseif (lyric == "dyo") then
            vowel = "o"
        elseif (lyric == "dha") then
            vowel = "a"
        elseif (lyric == "dhi") then
            vowel = "i"
        elseif (lyric == "dhu") then
            vowel = "u"
        elseif (lyric == "dhe") then
            vowel = "e"
        elseif (lyric == "dho") then
            vowel = "o"
        elseif (lyric == "bya") then
            vowel = "a"
        elseif (lyric == "byi") then
            vowel = "i"
        elseif (lyric == "byu") then
            vowel = "u"
        elseif (lyric == "bye") then
            vowel = "e"
        elseif (lyric == "byo") then
            vowel = "o"
-- ローマ字処理(複数文字(半濁音))
        elseif (lyric == "pya") then
            vowel = "a"
        elseif (lyric == "pyi") then
            vowel = "i"
        elseif (lyric == "pyu") then
            vowel = "u"
        elseif (lyric == "pye") then
            vowel = "e"
        elseif (lyric == "pyo") then
            vowel = "o"
-- 例外
        else
            vowel = lyric
        end
        io.write( '\n' )
        io.write( "    var myVowel" .. idx .. " = new MarkerValue(\"" .. vowel .. "\");\n" )
        io.write( "    V3Vowel.property(\"Marker\").setValueAtTime(" .. min .. ", myVowel" .. idx .. ");\n" )
        io.write( "    var curMarker = V3Vowel.property(\"Marker\").keyValue(" .. idx .. ");\n" )
        io.write( "    curMarker.cuePointName = myVowel" .. idx .. ";\n" )
        io.write( "    curMarker.eventCuePoint = true;\n" )
        io.write( "    V3Vowel.property(\"Marker\").setValueAtKey(" .. idx .. ",curMarker);\n" )

-- スライダー制御エフェクト
        if (vowel == "a") then
            key = 1
        elseif (vowel == "i") then
            key = 2
        elseif (vowel == "u") then
            key = 3
        elseif (vowel == "e") then
            key = 4
        elseif (vowel == "o") then
            key = 5
        elseif (vowel == "n") then
            key = 6
        else
            key = 0
        end
        
        par = idx / noteCount * 100
        par = math.floor(par)
        
        io.write( '    writeLn("");\n' )
        io.write( "    writeLn(\"Loading Vowel Data...\");\n" )
        io.write( "    writeLn(\"" .. par .. "%\");\n" )
        io.write( "    V3Vowel.effect(myGlobals.sliderEffectName)(myGlobals.sliderName).setValueAtTime(" .. min .. "," .. key .. ");\n" )
    end

    io.write( "  }\n" )
    io.write( '  writeLn("'.. pluginName ..'");\n' )
    io.write( '  writeLn("completed");\n' )
    io.write( '  writeLn("");\n' )
    io.write( "}\n" )
    io.write( '\n' )

-- 正常終了.
    return 0

end
